#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require File.expand_path(File.join(File.dirname(__FILE__), 'sidfm_vm_unix_agent.rb'))
require File.expand_path(File.join(File.dirname(__FILE__),  'centos_config.rb'))
require File.expand_path(File.join(File.dirname(__FILE__), 'application_patterns.rb'))

# CentOS 6/7/8, RHEL 5/6/7/8用エージェント

TestMode = if ARGV.size == 1 and ARGV[0] == 'test' then
             true
           else
             false
           end

OfflineMode = if ARGV.size >= 2 and ARGV[0] == '-local' then
                true
              else
                false
              end

class CentosAgent < SidfmVMUnixAgent

  include CentosConfig
  include ApplicationPatterns

  private

  def initialize(config_path)
    super(config_path, CMD_CENTOS, APPLICATIONS)
  end
 
  # pathが特定パッケージに含まれる場合には0、それ以外の
  # 場合には0以外を返す。
  def check_package_by_path(path)
    exec_command_status("#{@cmdset[:rpm][:cmd]} -qf \"#{path}\"")
  end

  def get_packages
    os_info = guess_os
    if (os_info[:id] == 'centos' or os_info[:id] == 'rhel') and /\A8(\.\d+)?\z/ =~ os_info[:version] then
      exec_command_stdout("rpm -qa --qf '%{name}-%{version}-%{release}.%{arch}.rpm %{modularitylabel}\n' | #{@cmdset[:sort][:cmd]}").split("\n")
    else
      exec_command_stdout("rpm -qa --qf '%{name}-%{version}-%{release}.%{arch}.rpm\n' | #{@cmdset[:sort][:cmd]}").split("\n")
    end
  end

  public

  # zipファイル内の指定したパスのファイル内容を取得する
  def extract_zip(path, arc_path)
    ret = exec_command_stdout("#{@cmdset[:unzip][:cmd]} -p \"#{path}\" \"#{arc_path}\"")
    to_ascii8bit(ret)
  end

  def check_init
    errors = check_required_packages
    errors.each do |e|
      STDERR.puts e[:message]
    end
    exit if errors.size > 0
    true
  end

  def guess_os
    ret = get_os_release
    ret = get_rhel_release unless ret
    ret
  end

  def get_package_informations
    get_packages
  end

end

if $0 == __FILE__ then

  require 'pp'

  o = CentosAgent.new(File.dirname(__FILE__) + '/centos_agent.yaml')
  pp o.check_init
  pp o.guess_os
  pp o.get_package_informations
  pp o.get_application_versions
  if OfflineMode then
    pp o.save_data(ARGV[1], ARGV[2])
  elsif !TestMode then
    pp o.register
    pp o.send_data
  end

end
